#ifndef UTILS_H
#define UTILS_H

#include <fstream>
#include <vector>

#include "definitions.h"

#ifdef MULTITHREAD_SUPPORT
#include <omp.h>
#endif


namespace CSA
{



struct Triple
{
  usint first;
  usint second;
  usint third;

  Triple();
  Triple(usint a, usint b, usint c);
};


std::streamoff fileSize(std::ifstream& file);
std::streamoff fileSize(std::ofstream& file);


std::ostream& operator<<(std::ostream& stream, pair_type data);


void readRows(std::ifstream& file, std::vector<std::string>& rows, bool skipEmptyRows);
void readPizzaChili(std::ifstream& file, std::vector<std::string>& patterns);

double readTimer();
usint  memoryUsage(); // Peak memory usage in kilobytes.


template<class T>
void
removeDuplicates(std::vector<T>* vec, bool parallel = true)
{
  if(vec == 0) { return; }
  #ifdef MULTITHREAD_SUPPORT
    if(parallel) { std::sort(vec->begin(), vec->end()); }
    else
    {
      #ifdef _GLIBCXX_PARALLEL
        std::sort(vec->begin(), vec->end(), __gnu_parallel::sequential_tag());
      #else
        std::sort(vec->begin(), vec->end(), mcstl::sequential_tag());
      #endif
    }
  #else
    std::sort(vec->begin(), vec->end());
  #endif
  vec->resize(std::unique(vec->begin(), vec->end()) - vec->begin());
}


} // namespace CSA


#endif // UTILS_H
